@@ -1,7 +1,7 @@
-/* pg_hint_plan/pg_hint_plan--1.7.0--1.7.1.sql */
+/* pg_hint_plan/pg_hint_plan--1.4.4--1.5.sql */
 
 -- complain if script is sourced in psql, rather than via CREATE EXTENSION
-\echo Use "ALTER EXTENSION pg_hint_plan UPDATE TO '1.7.1'" to load this file. \quit
+\echo Use "ALTER EXTENSION pg_hint_plan UPDATE TO '1.5'" to load this file. \quit
 
 SELECT pg_catalog.pg_extension_config_dump('hint_plan.hints','');
 SELECT pg_catalog.pg_extension_config_dump('hint_plan.hints_id_seq','');
